#ifndef BOOK_H
#define BOOK_H

#include <QTextDocument>
#include <QTextBlock>
#include <QHash>
#include <QList>
#include <QDate>
#include <QObject>

#include "epubgenerator.h"
#include "bookmark.h"
#include "annotation.h"

class BookProgress
{
public:
    BookProgress() : scrollProgress(0), progressSection(0), progressPage(0), percentage(-1.0){}
    //TODO: Check for correct values.
    void setProgress(int section, float p, int page = 0){
        progressSection = section;
        progressPage = page;
        percentage = p;
        
    }

    int getScroll(){return scrollProgress;}
    inline int getPage(){return progressPage;}
    int getSection(){return progressSection;}
    float getPercentage(){return percentage;}
    bool isValid(){ 
        return (percentage != -1);
    }
private:
    //Variables for saving book progress
    float percentage;
    int scrollProgress;
    int progressSection;
    int progressPage;
};

struct BookSectionData{
    QString *content;
    QString label;
    QString link;
    int TOCIndex;
};

class Book : public QObject
{
    Q_OBJECT
public:
    Book();
    Book(const QString &bookFilename, bool loadLater = false);
    ~Book();

    void openBook(const QString &bookFilename);
    void loadMetaData();
    void load();
    bool isLoaded();

    /*Returns length of book as number of characters.*/
    int getBookLength();

    /* Returns position of text fragment as int
     * If text is not found returns -1 
     */
    int getPositionInBook(const QString&);

    /* Returns paragraph at the given position
     * Position is a number between 0 and book length
     * If paragraph is not found returns an empty QString
     */
    QString getParagraphAtPosition(int, int&);

    /* These functions return the start position, lenght and
     * end position of the section in characters.
     */
    int getSectionStart(int);
    int getSectionLength(int);
    int getSectionEnd(int);

    /* Returns the section number of the section at given character position
     */
    int getSectionAtPosition(int);


    QString getFirstChapter();
    QString getNextChapter(QString&);
    QString getChapter(QString);

    QString getCover();
    QString getNextSection(int &number);
    QString getPreviousSection(int &number);
    const QString& getSection(int);
    int getSectionNumber(QString);
    const QString& getSectionName(int);
    QString getBook();
    QList<QString> getTOC();
    const QString getSectionOfTOCItem(int);
    int getSectionNumberOfTOCIndex(int);

    void setFilename(QString);
    QString getFilename();

    bool hasTOC();
    const QString& getTitle();
    const QString& getAuthor();
    const QString& getCategory();
    const QPixmap& getCoverImage();
    const QString& getSubject();
    const QString& getDescription();
    const QString& getPublishDate();
    int getNumberOfSections();

    unsigned int getLengthInWords();
    void setLengthInWords(unsigned int);

    BookProgress getProgress();
    void setProgress(BookProgress);

    QList<Bookmark> getBookmarks();
    void addBookmark(Bookmark);

    QList<Annotation> getAnnotations();
    void addAnnotation(Annotation);
    Annotation* getAnnotation(const QString&);

    bool isInLibrary();
    void changeLibraryStatus(bool);

    QNetworkAccessManager *getResourceManager();

private:
    QString title;
    QString author;
    QString publisher;
    QString description;
    QString subject;
    QString publishDate;
    QString category;
    QString copyright;
    QString filename;

    unsigned int lengthInWords;
    
    QList<QString> toc;
    QList<BookSectionData> sections;

//    QHash<QString, QString*> document;
//    QList<QString*> sections;

    QPixmap coverImage;

    QList<Bookmark> bookmarks;
    QList<Annotation> annotations;

    EpubGenerator *generator;

    BookProgress bookProgress;

    bool loaded;

    bool inLibrary;

    void setUpGenerator();


public slots:
    void setTitle(QString);
    void setAuthor(QString);
    void setPublisher(QString);
    void setDescription(QString);
    void setSubject(QString);
    void setPublishDate(QString);
    void setCategory(QString);
    void setCopyright(QString);

    void addChapter(QString, QString*);
    void addTOCItem(QString, QString);
    void addSection(QString*, QString);

    void addCoverImage(QPixmap);

signals:
    void coverImageAdded();

};

#endif
