#ifndef BOOKVIEW_H
#define BOOKVIEW_H

#include <QGraphicsView>
#include <QThread>
#include <QTimer>
#include <QSlider>
#include "libosso.h"
#include "book.h"
#include "shared.h"
#include "page.h"
#include "rendersettings.h"
#include "renderer.h"
#include "scrollablepage.h"
#include "bookprogressslider.h"

struct Section{
    int number;
    int contentHeight;
    int numPages;
    float pageMultiplier;
};

struct PositionInBook{
    Section section;
    int page;
    float multiplier;
};


class BookView : public QGraphicsView
{
    Q_OBJECT
public:
    BookView(QWidget *parent = 0);
    ~BookView();
    void setBook(Book*);

    void setFont(const QString&, int);
    void setNightMode(bool);
    void setViewMode(BookViewMode, Qt::WidgetAttribute);

    void addBookmark(QString, QColor);

protected:
    void mousePressEvent(QMouseEvent*);
    void mouseMoveEvent(QMouseEvent*);
    void mouseReleaseEvent(QMouseEvent*);
    void resizeEvent(QResizeEvent*);

private:
    Book *book;
    PositionInBook currentPosition;
    QList<PositionInBook> history;
    int currentSection;
    int currentPage;
    int sectionContentHeight;
    QPoint swipeStart;
    BookViewMode viewMode;
    QSizeF pageSize;
    RenderSettings renderSettings;

    QPixmap loadingPixmap();


    Page *pageWidgetCurrent;
    Page *pageWidgetNext;
    QGraphicsScene *scene;
    ScrollablePage *scrollablePage;
    BookProgressSlider *progressSlider;
    QGraphicsProxyWidget *sliderProxy;

    QHash<int, QHash<int, RenderTile*> > tiles;
    Renderer *renderer;
    void nextPage();
    void previousPage();
    QPixmap getCurrentPagePixmap();
    QPixmap getNextPagePixmap();
    QPixmap getPreviousPagePixmap();

    QTimer autoScrollTimer;
    const int autoScrollStep;
    int autoScrollBy; 

    QString annotationParagraph;

    bool isTap;
    int tapMove;

    void deleteTiles();


public slots:
    void goToSection(int, float multiplier = 0.0);
    void goToTOCIndex(int);
    void goToBookmark(int);
    void goToAnnotation(int);
    void backInHistory();
    void nextSection();
    void previousSection();
    void volumeUpPressed();
    void volumeDownPressed();
    void saveProgress();
    void addTile(RenderTile*);
    void contentHeight(int);
    void numberOfPages(int);
    void webPageLoaded(QWebPage*);
    void handleAutoScroll();
    void openContextMenu(const QPoint&);
    void addBookmark();
    void addAnnotation();
    void sectionScrolled(int);
    void sliderMoved();

signals:
    void requestRender(QString, int);

};

#endif
